#!/usr/bin/perl -I/usr/sausalito/perl
# $Id: am_vserver.pl
# Copyright 2007-2018 SOLARSPEED.NET. All rights reserved.
#
# Description:
#
# Script that runs through swatch, tests resources of a VPS and sets CCE objects 
# with status info about the VPS's. 
#

$whatami = "constructor";

use CCE;
use AM::Util;

my $cce = new CCE;
my $conf = '/var/lib/cobalt';

if ($whatami eq "handler") {
    $cce->connectfd();
}
else {
    $cce->connectuds();
}

# Setup array of return codes:
@return = ();

# Timespan in seconds between AM runs:
$span = '60';

# Check if there are VPS's that we need to parse:
&check_vps_list;

# Write new status info into CCE:
if ($running_vps_present == "1") {
    &swatch_push;
}

# Check prlctl:
&check_prlctl;

# Return status on CLI:
$back = "GREEN";
foreach $status (@return) {
    if ($status eq "RED") { $back = "RED"; }
    if ($status eq "YELLOW") { $back = "YELLOW"; }
}
my %am_states = am_get_statecodes();
if ($back eq "RED") {
    print "[[base-vserver.AM-VPS-RED]]";
    exit $am_states{AM_STATE_RED};
}
if ($back eq "YELLOW") {
    print "[[base-vserver.AM-VPS-YELLOW]]";
    exit $am_states{AM_STATE_YELLOW};
}
if ($back eq "GREEN") {
    print "[[base-vserver.AM-VPS-GREEN]]";
    exit $am_states{AM_STATE_GREEN};
}

$cce->bye('SUCCESS');
exit(0);

sub check_prlctl {
    $prlcheck = `/usr/bin/prlctl list -a`;
    chomp($prlcheck);
    if ($line =~ /^Login failed: Unable to connect to Virtuozzo.(.*)$/) {
        system("/usr/bin/systemctl enable prl-disp.service");
        system("/usr/bin/systemctl restart prl-disp.service");
    }
}

sub check_vps_list {
    $running_vps_present = '0';
    if (( -d "/vz/private") && ( -f "/proc/vz/veinfo")) {
        $running_vps_present = `/usr/sbin/vzlist -a --no-header | grep -v stopped | grep -v mounted | wc -l`;
        chomp($running_vps_present);
        if ($running_vps_present gt '1') {
            $running_vps_present = '1';
        }
    }
}

sub swatch_push {

    %VPSERR = {};

    # Generate vzlist-dump:
    $vzlistdump = `/usr/sbin/vzlist -a -H -o ctid,kmemsize.f,lockedpages.f,privvmpages.f,shmpages.f,numproc.f,physpages.f,vmguarpages.f,oomguarpages.f,numtcpsock.f,numflock.f,numpty.f,numsiginfo.f,tcpsndbuf.f,tcprcvbuf.f,othersockbuf.f,dgramrcvbuf.f,numothersock.f,dcachesize.f,numfile.f,numiptent.f,swappages.f | sed 's/^[ \t]*//;s/[ \t]*\$//'`;

    # Pull the vzlist-dump into a Matrix:
    @vzlistdump_arr = split(/\n/, $vzlistdump);
    foreach $line (@vzlistdump_arr) {
        chomp($line);
        # skip blank lines
        next if $line =~ /^\s*$/;
        # Split at spaces:
        my (@row) = split (/\s+/, $line);
        # Get VPSID from first element:
        $vpsid = shift @row;
        # Walk through rest and sum the numbers up:
        $total_err = '0';
        foreach $err (@row) { 
            $total_err = $total_err+$err;
        }
        $VPSERR->{$vpsid} = $total_err;
        # Feed info into CCE
        &feedthemonster($vpsid, $total_err);
    }
}

sub feedthemonster {

    my $f_vpsid = shift;
    my $f_errs = shift;

    # Get Vserver data for that VPS:
    ($vpsOID) = $cce->find('Vserver', {'vpsid' => $f_vpsid});

    if ($vpsOID eq "") {
        # Not in CODB, aborting loop.
        return;
    }

    # Is the VPS running?
    $is_running = `/usr/sbin/vzlist $f_vpsid -H -o ctid,status|grep running|wc -l`;
    chomp($is_running);

    $failcnt_general = 'N';

    if ($is_running eq '1') {
        #print "Object already present in CCE. Updating it.\n";
        ($ok, $Vserver) = $cce->get($vpsOID);

        # Has the number of errors increased since last run?
        if ($f_errs eq "0") {
            $failcnt_general = 'G';
            push @return, 'GREEN';
        }
        elsif ($f_errs gt $Vserver->{'failcnt'}) {
            # Yes, raise active error:
            $failcnt_general = 'R';
            push @return, 'RED';
        }
        elsif ($f_errs eq $Vserver->{'failcnt'}) {
            $event_time = $Vserver->{'failcnt_lastevent'};
            if ($event_time+900 gt time()) {
                $failcnt_general = 'Y';
                push @return, 'YELLOW';
            }
            else {
                $failcnt_general = 'G';
                push @return, 'GREEN';
            }
        }
        else {
            # Nope:
            $failcnt_general = 'G';
            push @return, 'GREEN';
        }

        ($ok) = $cce->set($vpsOID, '', {
                                            'failcnt' => $f_errs,
                                            'failcnt_lastevent' => $Vserver->{'failcnt'},
                                            'failcnt_general' => $failcnt_general,
                                            'failcnt_time' => time()
                                            });
    }
    else {
        # VPS is not running.
        #print "VPS is not running.\n";
        ($ok) = $cce->set($vpsOID, '', {
                                            'failcnt' => '0',
                                            'failcnt_lastevent' => '0',
                                            'failcnt_general' => $failcnt_general,
                                            'failcnt_time' => time(),
                                            });
        push @return, 'GREEN';
    }
}

